"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const report_1 = require("stryker-api/report");
const fs = require("mz/fs");
const chalk = require("chalk");
const _ = require("lodash");
const tempy = require('tempy');
const path = require("path");
class default_1 {
    constructor(_, baseline = read(), write = console.log) {
        this.baseline = baseline;
        this.write = write;
        this.filename = path.join(tempy.directory(), 'stryker.baseline.js');
    }
    onAllMutantsTested(results) {
        this.surviving = results.filter(_ => _.status == report_1.MutantStatus.Survived);
        let fresh = _.differenceWith(this.surviving, this.baseline, (lhs, rhs) => this.compare(lhs, rhs));
        let gone = _.differenceWith(this.baseline, this.surviving, (lhs, rhs) => this.compare(lhs, rhs));
        if (fresh.length == 0) {
            if (this.surviving.length == this.baseline.length) {
                this.write(chalk.green(chalk.bold('Good job!'), 'At least you did not introduce new mutants.'));
            }
            else {
                this.write(chalk.green(chalk.bold('Great job!'), 'You killed some mutants that someone other than you created.'));
                this.write('You should update the baseline file right now!');
            }
        }
        else {
            fresh.forEach(_ => this.write(_.sourceFilePath));
            if (gone.length == 0) {
                this.write(chalk.red(chalk.bold('Shame on you!'), 'New mutants survived. Let\'s go kill them!'));
                this.write('If it wasn\'t your fault or you just can\'t fix them you should update the baseline file.');
            }
            else {
                this.write(chalk.yellow(chalk.bold('Mixed feelings.'), 'You killed some mutants but also introduced some.'));
                this.write('Mind updating the baseline file will let them live forever...');
            }
        }
    }
    wrapUp() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            this.write(`Saved a new baseline file: ${this.filename}`);
            yield fs.writeFile(this.filename, JSON.stringify(this.surviving));
        });
    }
    compare(lhs, rhs) {
        return lhs.mutatedLines == rhs.mutatedLines
            && lhs.originalLines == rhs.originalLines
            && lhs.mutatorName == rhs.mutatorName
            && lhs.sourceFilePath == rhs.sourceFilePath;
    }
}
exports.default = default_1;
function read(filename = 'stryker.baseline.js') {
    return fs.existsSync(filename) ? JSON.parse(fs.readFileSync(filename, 'utf-8')) : [];
}
exports.read = read;
//# sourceMappingURL=BaselineReporter.js.map